@@ -133,6 +133,55 @@ pub fn white() -> ColorArray {
         .to_arr()
 }
 
+#[inline]
+pub fn default_light_black() -> ColorArray {
+    ColorBuilder::from_hex(String::from("#6B6B6B"), Format::SRGB0_1)
+        .unwrap()
+        .to_arr()
+}
+#[inline]
+pub fn default_light_blue() -> ColorArray {
+    ColorBuilder::from_hex(String::from("#82B8C8"), Format::SRGB0_1)
+        .unwrap()
+        .to_arr()
+}
+#[inline]
+pub fn default_light_cyan() -> ColorArray {
+    ColorBuilder::from_hex(String::from("#93D3C3"), Format::SRGB0_1)
+        .unwrap()
+        .to_arr()
+}
+#[inline]
+pub fn default_light_green() -> ColorArray {
+    ColorBuilder::from_hex(String::from("#AAC474"), Format::SRGB0_1)
+        .unwrap()
+        .to_arr()
+}
+#[inline]
+pub fn default_light_magenta() -> ColorArray {
+    ColorBuilder::from_hex(String::from("#C28CB8"), Format::SRGB0_1)
+        .unwrap()
+        .to_arr()
+}
+#[inline]
+pub fn default_light_red() -> ColorArray {
+    ColorBuilder::from_hex(String::from("#C55555"), Format::SRGB0_1)
+        .unwrap()
+        .to_arr()
+}
+#[inline]
+pub fn default_light_white() -> ColorArray {
+    ColorBuilder::from_hex(String::from("#F8F8F8"), Format::SRGB0_1)
+        .unwrap()
+        .to_arr()
+}
+#[inline]
+pub fn default_light_yellow() -> ColorArray {
+    ColorBuilder::from_hex(String::from("#FECA88"), Format::SRGB0_1)
+        .unwrap()
+        .to_arr()
+}
+
 #[inline]
 pub fn split() -> ColorArray {
     ColorBuilder::from_hex(String::from("#292527"), Format::SRGB0_1)
