@@ -1,3 +1,4 @@
+mod builtin;
 mod runtime;
 
 use crate::context::Context;
@@ -5,6 +6,8 @@ use librashader_common::{Size, Viewport};
 use librashader_presets::ShaderFeatures;
 use std::sync::Arc;
 
+pub type Filter = String;
+
 /// A brush for applying RetroArch filters.
 #[derive(Default)]
 pub struct FiltersBrush {
@@ -15,11 +18,11 @@ pub struct FiltersBrush {
 
 impl FiltersBrush {
     #[inline]
-    pub fn update_filters(&mut self, ctx: &Context, filter_paths: &[String]) {
+    pub fn update_filters(&mut self, ctx: &Context, filters: &[Filter]) {
         self.filter_chains.clear();
         self.filter_intermediates.clear();
 
-        if filter_paths.is_empty() {
+        if filters.is_empty() {
             return;
         }
 
@@ -33,18 +36,54 @@ impl FiltersBrush {
             return;
         }
 
-        for path in filter_paths {
-            tracing::debug!("Loading filter {}", path);
+        for filter in filters {
+            let configured_filter = filter.to_lowercase();
+            match configured_filter.as_str() {
+                "newpixiecrt" | "fubax_vr" => {
+                    tracing::debug!("Loading builtin filter {}", configured_filter);
 
-            match crate::components::filters::runtime::FilterChain::load_from_path(
-                path,
-                ShaderFeatures::NONE,
-                &ctx.device,
-                &ctx.queue,
-                None,
-            ) {
-                Ok(f) => self.filter_chains.push(f),
-                Err(e) => tracing::error!("Failed to load filter {}: {}", path, e),
+                    let builtin_filter = match configured_filter.as_str() {
+                        "newpixiecrt" => builtin::newpixiecrt,
+                        "fubax_vr" => builtin::fubaxvr,
+                        _ => {
+                            continue;
+                        }
+                    };
+
+                    match builtin_filter() {
+                        Ok(shader_preset) => {
+                            match crate::components::filters::runtime::FilterChain::load_from_preset(
+                                shader_preset,
+                                &ctx.device,
+                                &ctx.queue,
+                                None,
+                            ) {
+                                Ok(f) => self.filter_chains.push(f),
+                                Err(e) => tracing::error!("Failed to load builtin filter {}: {}", configured_filter, e),
+                            }
+                        },
+                        Err(e) => {
+                            println!("{:?}", e);
+                            tracing::error!("Failed to build shader preset from builtin filter {}: {}", configured_filter, e)
+                        },
+                    }
+                }
+                _ => {
+                    tracing::debug!("Loading filter {}", filter);
+
+                    match crate::components::filters::runtime::FilterChain::load_from_path(
+                        filter,
+                        ShaderFeatures::NONE,
+                        &ctx.device,
+                        &ctx.queue,
+                        None,
+                    ) {
+                        Ok(f) => self.filter_chains.push(f),
+                        Err(e) => {
+                            tracing::error!("Failed to load filter {}: {}", filter, e)
+                        }
+                    }
+                }
             }
         }
 
@@ -168,7 +207,7 @@ impl FiltersBrush {
                 Viewport::new_render_target_sized_origin(dst_output_view, None).unwrap();
 
             // Framecount should be added forever: https://github.com/raphamorim/rio/issues/753
-            let _ = self.framecount.wrapping_add(1);
+            self.framecount = self.framecount.wrapping_add(1);
             if let Err(err) = filter.frame(
                 filter_src_texture,
                 &dst_viewport,
